Package: poorman
Type: Package
Title: A Poor Man's Dependency Free Recreation of 'dplyr'
Version: 0.2.8
Authors@R: c(
    person("Nathan", "Eastwood", "", "nathan.eastwood@icloud.com",
           role = c("aut", "cre")),
    person("Etienne", "Bacher", , "etienne.bacher@protonmail.com", role = "ctb",
           comment = c(ORCID = "0000-0002-9271-5075")))
Maintainer: Nathan Eastwood <nathan.eastwood@icloud.com>
Description: A replication of key functionality from 'dplyr' and the wider 'tidyverse' using only 'base'.
URL: https://nathaneastwood.github.io/poorman/,
        https://github.com/nathaneastwood/poorman
BugReports: https://github.com/nathaneastwood/poorman/issues
Depends: R (>= 3.3)
Suggests: knitr, rmarkdown, roxygen2, tinytest
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.3
VignetteBuilder: knitr
Language: en-GB
NeedsCompilation: no
Packaged: 2026-07-30 16:04:53 UTC; nathan
Author: Nathan Eastwood [aut, cre],
  Etienne Bacher [ctb] (ORCID: <https://orcid.org/0000-0002-9271-5075>)
Repository: CRAN
Date/Publication: 2026-07-31 00:00:36 UTC
Built: R 4.5.2; ; 2026-07-31 01:41:03 UTC; unix
