# Generated by roxygen2: do not edit by hand

S3method(arrange,data.frame)
S3method(default_missing,data.frame)
S3method(default_missing,default)
S3method(distinct,data.frame)
S3method(distinct,grouped_df)
S3method(fill,data.frame)
S3method(fill,grouped_df)
S3method(filter,data.frame)
S3method(filter,grouped_df)
S3method(glimpse,data.frame)
S3method(glimpse,default)
S3method(group_by,data.frame)
S3method(group_by_drop_default,default)
S3method(group_by_drop_default,grouped_df)
S3method(group_data,data.frame)
S3method(group_data,grouped_df)
S3method(mutate,data.frame)
S3method(mutate,grouped_df)
S3method(nest_by,data.frame)
S3method(nest_by,grouped_df)
S3method(recode,character)
S3method(recode,factor)
S3method(recode,numeric)
S3method(relocate,data.frame)
S3method(rename,data.frame)
S3method(rename_with,data.frame)
S3method(replace_na,data.frame)
S3method(replace_na,default)
S3method(slice,data.frame)
S3method(slice,grouped_df)
S3method(slice_head,data.frame)
S3method(slice_head,grouped_df)
S3method(slice_max,data.frame)
S3method(slice_max,grouped_df)
S3method(slice_min,data.frame)
S3method(slice_min,grouped_df)
S3method(slice_sample,data.frame)
S3method(slice_sample,grouped_df)
S3method(slice_tail,data.frame)
S3method(slice_tail,grouped_df)
S3method(summarise,data.frame)
S3method(summarise,grouped_df)
S3method(summarize,data.frame)
S3method(summarize,grouped_df)
S3method(transmute,data.frame)
S3method(transmute,grouped_df)
S3method(ungroup,data.frame)
S3method(ungroup,grouped_df)
S3method(union_all,data.frame)
S3method(union_all,default)
export("%>%")
export(across)
export(add_count)
export(add_tally)
export(all_of)
export(anti_join)
export(any_of)
export(arrange)
export(between)
export(bind_cols)
export(bind_rows)
export(case_when)
export(coalesce)
export(column_to_rownames)
export(contains)
export(count)
export(cumall)
export(cumany)
export(cume_dist)
export(cummean)
export(cur_column)
export(cur_data)
export(cur_data_all)
export(cur_group)
export(cur_group_id)
export(cur_group_rows)
export(dense_rank)
export(desc)
export(distinct)
export(ends_with)
export(everything)
export(fill)
export(filter)
export(first)
export(full_join)
export(glimpse)
export(group_by)
export(group_by_drop_default)
export(group_cols)
export(group_data)
export(group_indices)
export(group_keys)
export(group_rows)
export(group_size)
export(group_split)
export(group_vars)
export(groups)
export(has_rownames)
export(if_all)
export(if_any)
export(if_else)
export(inner_join)
export(lag)
export(last)
export(last_col)
export(lead)
export(left_join)
export(lst)
export(matches)
export(min_rank)
export(mutate)
export(n)
export(n_distinct)
export(n_groups)
export(na_if)
export(near)
export(nest_by)
export(nth)
export(ntile)
export(num_range)
export(peek_vars)
export(percent_rank)
export(pivot_longer)
export(pivot_wider)
export(pull)
export(recode)
export(recode_factor)
export(relocate)
export(remove_rownames)
export(rename)
export(rename_with)
export(replace_na)
export(right_join)
export(row_number)
export(rowid_to_column)
export(rownames_to_column)
export(select)
export(semi_join)
export(slice)
export(slice_head)
export(slice_max)
export(slice_min)
export(slice_sample)
export(slice_tail)
export(starts_with)
export(summarise)
export(summarize)
export(tally)
export(transmute)
export(ungroup)
export(union_all)
export(unite)
export(where)
export(with_groups)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(utils,head)
